package main

import (
	"context"
	"errors"
	"fmt"
	"io"
	"log/slog"
	"net/http"
	"os"
	"os/signal"
	"syscall"
)

func main() {
	ctx := context.Background()

	server := http.Server{
		Addr: ":9234",
		Handler: http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
			fmt.Println(r.Method, r.URL)
			body, _ := io.ReadAll(r.Body)
			fmt.Println(string(body))

			w.WriteHeader(http.StatusOK)
		}),
	}

	idleConnsClosed := make(chan struct{})
	go func() {
		waitForShutdownSignal(ctx)

		slog.Info("received shutdown signal")

		ctxTimeout, cancel := context.WithTimeout(ctx, 10)
		defer cancel()

		if err := server.Shutdown(ctxTimeout); err != nil {
			slog.Warn("shutdown failed", slog.Any("error", err))
		}

		close(idleConnsClosed)
	}()

	slog.Info("Server is listening on the addr "+server.Addr, slog.String("addr", server.Addr))

	if err := server.ListenAndServe(); err != nil && !errors.Is(err, http.ErrServerClosed) {
		slog.Error("ListenAndServe failed", slog.Any("err", err))
		os.Exit(1)
	}

	<-idleConnsClosed
}

func waitForShutdownSignal(ctx context.Context) {
	ctx, stop := signal.NotifyContext(ctx, os.Interrupt, syscall.SIGTERM)
	defer stop()
	<-ctx.Done()
}
